!!ARBfp1.0
OPTION NV_fragment_program2;

# Copyright (c) 2005  Institute for Visualization and Interactive
# Systems, University of Stuttgart, Germany
#
# This source code is distributed as part of the single-pass volume
# rendering project. Details about this project can be found on the
# project web page at http://www.vis.uni-stuttgart.de/eng/research/
# fields/current/spvolren. This file may be distributed, modified,
# and used free of charge as long as this copyright notice is
# included in its original form. Commercial use is strictly
# prohibited.
#
# Filename: volume_sm3.fp

#! MEASURE = 2
#! VOLUME = 0
#! TRANSFERFUNCTION = 1
# BACKGROUND = 2
# JITTER = 3
#! VISIBILITY = 4

# r = step
# g = gradient scale
# b = gradient offset
# a = z-value of background plane
PARAM params = program.local[0];

# r = texture coordinate scale
# g = number of iterations
# b = isovalue of isosurface
PARAM params2 = program.local[1];
PARAM center = program.local[2];
PARAM texMax = program.local[3];

PARAM vport = program.local[4];

PARAM scaleFactors = program.local[5];

PARAM clipValue = program.local[7];

PARAM clipPlaneN = program.local[8]; # Plane normal
PARAM clipPlaneA = program.local[9]; # Point on plane

PARAM viewPlaneN0 = program.local[10];
PARAM viewPlaneA0 = program.local[11];

PARAM viewPlaneN1 = program.local[12];
PARAM viewPlaneA1 = program.local[13];

PARAM viewPlaneN2 = program.local[14];
PARAM viewPlaneA2 = program.local[15];

PARAM viewPlaneColor0 = program.local[16];
PARAM viewPlaneColor1 = program.local[17];
PARAM viewPlaneColor2 = program.local[18];

PARAM edgeColor = program.local[19];

TEMP geomDir;
TEMP geomPos;
TEMP texblen;
TEMP diffVec;
TEMP scalar;
TEMP camera;
TEMP normal;
TEMP temp1;
TEMP temp2;
TEMP temp;
TEMP pos;
TEMP src;
TEMP dst;
TEMP dir;
TEMP tex;

TEMP edgePassed;
#TEMP edgeColor;
MOV edgePassed, 0;

#MOV edgeColor, 0;
#MOV edgeColor.x, .2;
#MOV edgeColor.y, .5;
#MOV edgeColor.z, .2;

TEMP measureColor;
MOV measureColor, 0;
MOV measureColor.x, .8;
MOV measureColor.y, .8;
MOV measureColor.z, .2;
MOV measureColor.a, .5;

TEMP viewPlanePassed;
MOV viewPlanePassed, 0; # XYZ=012

# Compute the ray's starting point
MOV geomPos, fragment.texcoord[0];
MUL pos, geomPos, scaleFactors;
MOV pos.a, 0.0;

# Compute the camera position by translating the origin to the center of the
# volume
MOV camera, state.matrix.modelview.invtrans.row[3];

# Compute the ray direction
SUB geomDir, geomPos, camera;

# Normalize the direction (done manually instead of with NRM to improve
# accuracy)
NRM geomDir, geomDir;
#DP3 geomDir.w, geomDir, geomDir;
#RSQ geomDir.w, geomDir.w;
#MUL geomDir, geomDir, geomDir.w;
#MOV geomDir.w, 0.0;
MUL dir, geomDir, scaleFactors;

# Initialize scalar value
# RGB = gradient, alpha = scalar value
TXL scalar, pos, texture[0], 3D;
MOV scalar.g, scalar.a;
MOV scalar.a, 0.0;

MOV scalar.b, 0.0;

# Initialize destination color
MOV dst, 0.0;

# Move one step forward
MAD pos, dir, params.r, pos;

REP params2.g;
	REP params2.g;

		# Lookup new scalar value
		TXL tex, pos, texture[0], 3D;           # tex = texture3D[pos];
		MOV scalar.r, tex.a;                    # scalar.r = tex.a;
		
		# Lookup color in pre-int texture
		TXL src, scalar, texture[1], 2D;        # src = texture2D[scalar];
		
		
		# TODO Early ray termination (no gain of speed so far)
		SUBC texblen.r, 1.0, dst.a; # texblen.r = 1.0 - dst.a;
		BRK (LE.x);

		#
		# Show edges
		#
		MOV edgePassed.y, 0;
		MOVC temp1.x, edgePassed.x;		
		IF EQ.x;
			# X
			SUBC temp1.x, pos.x, .005;
			IF LE.x;
				ADD edgePassed.y, edgePassed.y, 1;	
			ELSE;
				SUBC temp1, pos.x, .995;
				IF GT.x;
					ADD edgePassed.y, edgePassed.y, 1;					
				ENDIF;
			ENDIF;
			
			# Y
			SUBC temp1.x, pos.y, .005;
			IF LE.x;
				ADD edgePassed.y, edgePassed.y, 1;	
			ELSE;
				SUBC temp1, pos.y, .995;
				IF GT.x;
					ADD edgePassed.y, edgePassed.y, 1;					
				ENDIF;
			ENDIF;

			# Z
			SUBC temp1.x, pos.z, .005;
			IF LE.x;
				ADD edgePassed.y, edgePassed.y, 1;	
			ELSE;
				SUBC temp1, pos.z, .995;
				IF GT.x;
					ADD edgePassed.y, edgePassed.y, 1;					
				ENDIF;
			ENDIF;

			SUBC temp1.x, edgePassed.y, 1;
			IF GT.x;
				MAD_SAT dst, edgeColor, texblen.r, dst;
				MOV edgePassed.x, 1;
			ENDIF;
		ENDIF;
		
		#
		# Show view planes
		#
		
		# Sagittal
		MOVC temp1.x, viewPlanePassed.x;
		IF EQ.x;
			SUB temp1, pos, viewPlaneA0;
			DP3 temp1.r, temp1, viewPlaneN0;		
			SUBC temp1.r, temp1.r, .005;
			IF LE.x;
				SUB temp1, pos, viewPlaneA0;
				DP3C temp1.r, temp1, viewPlaneN0;
				IF GE.x;
					MAD_SAT dst, viewPlaneColor0, texblen.r, dst;
					MOV viewPlanePassed.x, 1;
				ENDIF;
			ENDIF;
		ENDIF;

		# Coronal
		MOVC temp1.x, viewPlanePassed.y;
		IF EQ.x;
			SUB temp1, pos, viewPlaneA1;
			DP3 temp1.r, temp1, viewPlaneN1;		
			SUBC temp1.r, temp1.r, .005;
			IF LE.x;
				SUB temp1, pos, viewPlaneA1;
				DP3C temp1.r, temp1, viewPlaneN1;
				IF GE.x;
					MAD_SAT dst, viewPlaneColor1, texblen.r, dst;
					MOV viewPlanePassed.y, 1;
				ENDIF;
			ENDIF;
		ENDIF;
		
		# Transversal
		MOVC temp1.x, viewPlanePassed.z;
		IF EQ.x;
			SUB temp1, pos, viewPlaneA2;
			DP3 temp1.r, temp1, viewPlaneN2;		
			SUBC temp1.r, temp1.r, .005;
			IF LE.x;
				SUB temp1, pos, viewPlaneA2;
				DP3C temp1.r, temp1, viewPlaneN2;
				IF GE.x;
					MAD_SAT dst, viewPlaneColor2, texblen.r, dst;
					MOV viewPlanePassed.z, 1;
				ENDIF;
			ENDIF;
		ENDIF;
		
		# Check clipping plane
		SUB temp1, pos, clipPlaneA;
		DP3C temp1.r, temp1, clipPlaneN;
		IF GE.x;
			# Check visibility data
			TXL tex, pos, texture[4], 3D;
			SUBC tex.r, tex.r, clipValue.r; 
			IF GE.x;
				# Perform blending
				MAD_SAT dst, src, texblen.r, dst;       # dst += src * texblen.r;
				
				# Measures
				TXL src, pos, texture[2], 3D;
				MUL_SAT src.r, src.r, 1024; # saturate to 1
				MAD dst, measureColor, src.r, dst;
			ENDIF;
		ENDIF;

		# Move one step forward
		MAD pos, dir, params.r, pos;

		# Terminate loop if outside volume
		SGE temp1, pos, 0.0;
		SLE temp2, pos, texMax;
		DP3 temp1.r, temp1, temp2;
		SEQC temp1.r, temp1.r, 3;
		BRK (EQ.x);

		# Save current scalar value
		MOV scalar.g, scalar.r;

	ENDREP;

	BRK (LE.x);

ENDREP;

# Compute the normal of the background plane (this is just the negative view
# direction which initially is (0, 0, -1))
#MOV normal, state.matrix.modelview.row[2];

# Compute the plane constant (we want the plane to be located in the volume
# center)
#DP3 temp1.r, normal, center;

# Move the plane behind the volume: d' = <n,(x - l n)> = <n, x> - l <n, n>;
# l = 0.71 is chosen since it is greater than half the cube diagonal 
#DP3 temp1.g, normal, normal;
#MAD temp1.r, temp1.g, -.71, temp1.r;

# Compute ray parameter
#DP3 temp1.g, normal, geomPos;
#SUB temp1.g, temp1.r, temp1.g;
#DP3 temp1.b, normal, geomDir;
#DIV temp.r, temp1.g, temp1.b;

# Compute ray/plane intersection
#MAD temp.rgb, temp.r, geomDir, geomPos;

# Compute the difference vector
#SUB diffVec, temp, center;

# Compute the texture coordinates
#DP3 temp.r, diffVec, state.matrix.modelview.row[0];
#DP3 temp.g, diffVec, state.matrix.modelview.row[1];
#MUL temp.rg, temp, params2.r;

# Center background image
#ADD temp.rg, temp, .5;

# Look up the texel value
#TEX temp.rgb, temp, texture[2], 2D;
MOV temp, .4;
MOV temp.a, 1.0;

# Blend the background pixel
SUB texblen, 1.0, dst.a;
MAD dst, temp, texblen.x, dst;

# Write the output color
MOV result.color, dst;

#MOV result.color, geomPos;#fragment.texcoord[0];

END
	